@@ -1,9 +1,5 @@
-#!/bin/sh
+#!@TCLSH@
 # -*- coding: utf-8; mode: tcl; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- vim:fenc=utf-8:filetype=tcl:et:sw=4:ts=4:sts=4
-# Run the Tcl interpreter \
-exec @TCLSH@ "$0" "$@"
-# portmirror.tcl
-# $Id: portmirror.tcl 71011 2010-08-29 23:14:13Z raimue@macports.org $
 #
 # Copyright (c) 2002-2007 The MacPorts Project.
 # All rights reserved.
@@ -11,12 +7,10 @@ exec @TCLSH@ "$0" "$@"
 # Updates the distfiles to current distfiles by deleting old stuff.
 # Uses the database.
 
-source [file join "@macports_tcl_dir@" macports1.0 macports_fastload.tcl]
 package require macports
 package require Pextlib
 
 # Globals
-global distfiles_filemap
 array set ui_options        [list]
 array set global_options    [list]
 array set global_variations [list]
@@ -47,7 +41,7 @@ proc iterate_distfiles_r {func root} {
 #           the path as its parameter)
 proc iterate_distfiles {func} {
     global macports::portdbpath
-    iterate_distfiles_r $func [file join ${macports::portdbpath} distfiles]
+    iterate_distfiles_r $func [file join ${portdbpath} distfiles]
 }
 
 # Check if the file is in the map and delete it otherwise.
@@ -60,9 +54,9 @@ proc iterate_walker {path} {
 }
 
 # Open the database
-proc open_database args {
+proc open_database {args} {
     global macports::portdbpath distfiles_filemap
-    set path [file join ${macports::portdbpath} distfiles_mirror.db]
+    set path [file join ${portdbpath} distfiles_mirror.db]
     if {[file exists $path]} {
         filemap open distfiles_filemap $path readonly
     } else {
@@ -71,18 +65,18 @@ proc open_database args {
 }
 
 # Close the database
-proc close_database args {
+proc close_database {args} {
     global distfiles_filemap
     filemap close distfiles_filemap
 }
 
 # Standard procedures
-proc print_usage args {
+proc print_usage {args} {
     global argv0
     puts "Usage: $argv0"
 }
 
-if {[expr $argc > 0]} {
+if {$argc > 0} {
     print_usage
     exit 1
 }
