@@ -12,7 +12,8 @@ set_dir
 port_index
 port_config $path
 file copy -force $path/statefile $work_dir/.macports.statefile-version2-outdated.state
-port_desroot $path
+file attributes $work_dir/.macports.statefile-version2-outdated.state -permissions 0664
+port_destroot $path
 port_clean $path
 
 proc state_v2_out {warn} {
@@ -32,7 +33,7 @@ test warning_check {
     Regression test for statefile-v2-outdated discard prev version.
 } -body {
     state_v2_out yes
-} -result "portfile changed since last build; discarding previous state."
+} -result "portfile for statefile-version2-outdated changed since last build; discarding previous state."
 
 test output_check {
     Regression test for statefile-v2-outdated output.
